

function [CGMMparams, LSGMMparams] = trainCGMM( X, Y, varargin)
%
% function [CGMMparams, LSGMMparams] = trainCGMM( X, Y,'Parameter1',Value1,'Parameter2',Value2,...);
%
% Train a CGMM conversion
%
% Input:
%
% X = spectral features of the source speaker (each row is an input vector) 
% Y = spectral features of the target speaker (each row is an input vector) 
% 
% Parameters:
% diagonalCov (default 1) train a diagonal cov. matrices, non diagonal
% conversion is not supported.
% maxiterations (default 1e5) - for training GMM of the source spectra
% LLth (default 0.00001) - relative change in the likelihood function) used as a threshold for the stopping critiria for training the GMM of the source spectra. 
% overwrite (default true) - retrain and not use pre-saved results
% cepsVarOrdStop (default 12) - amount of elements to be enhanced
%
% Output:
%
% CGMMparams = conversion parameters trained by CGMM used as a simple
% GBB-based linear conversion
% LSGMMparams = conversion parameters trained by LS-GMM, evaluated as a by-product of the CGMM    
%  
% Version 1.0
% copyright by Hadas Benisty (2012)
% Technion
% contact hadasbe@tx.technion.ac.il
%

pars.diagonalCov = 1;
pars.maxiterations = 1e5;
pars.LLth=0.00001;

pars.overwrite = true;
pars.cepsVarOrdStop = 12;
pars=extractpars(varargin,pars);





GMMsource = trainGMM(X, pars);

P_ls = transpose(expectationStep(GMMsource, X.'));
if pars.diagonalCov
    
    [~, ord] = size(X);
    al2 = zeros(ord, 1);
    for j=1:ord
        D_lsj = getLSGMMmatrixD(GMMsource, X, P_ls, pars, j);
        Aj = [P_ls, D_lsj(:,:)];
        
        Bj = mulByM(Aj);
        Myp = mulByM(Y(:,j));
        al2(j)= sum(Myp.^2);
        
        [U,~,Xls,C,S] = gsvd(Aj,Bj);
       
        b_tildj = U'*Y(:,j);
        Daj = (diag(C));
        Dbj = (diag(S));
        
        bj_tild_partial=b_tildj(1:size(Bj,2));
        xj = pinv(Aj)*Y(:,j);
        
        
        ni(:,j) = xj(1:pars.m);
        lambda(:,j) = xj(pars.m+1:end);
        
        if j>pars.cepsVarOrdStop
            ni_ls(:,j) = ni(:,j);
            lambda_ls(:,j) = lambda(:,j);
        else
            
            lam_final(j) = gridSearchSol(b_tildj(1:size(Bj,2)), Daj,Dbj,Aj,Bj,al2(j),Xls,Y(:,j),pars.m, -(Daj(1)./Dbj(1))^2*1.1,0);
            
            xij_tild = diag(Daj./(Daj.^2+lam_final(j) *Dbj.^2))*bj_tild_partial;
            xij = (Xls')\xij_tild;
            
            
            
            ni_ls(:,j) = xij(1:pars.m);
            lambda_ls(:,j) = xij(pars.m+1:end);
            LSD_LSGMM = ((Aj*xj-Y(:,j))'*(Aj*xj-Y(:,j)));
            GV_LSGMM =  ((Bj*xj)'*Bj*xj);
            
            LSD_CGMM = ((Aj*xij-Y(:,j))'*(Aj*xij-Y(:,j)));
            GV_CGMM =  ((Bj*xij)'*Bj*xij);
            disp(sprintf('Element no. %d:', j));
            disp(sprintf('LS-GMM:\tEuc. Dist(%d) = %f;\tGV(%d) = %f', j,LSD_LSGMM, j, GV_LSGMM ));
            disp(sprintf('CGMM:\tEuc. Dist(%d) = %f;\tGV(%d) = %f', j,LSD_CGMM, j, GV_CGMM ));
            
        end
        
        
        clear U V Xls C S;
    end
    disp(1);
    LSGMMparams(1:pars.m)=struct('a',0,'u',[],'E',[],'v',[],'R',[]);
    for k=1:pars.m
        % transform: x->y
        LSGMMparams(k).a=GMMsource(k).a; % ord(Wk)
        LSGMMparams(k).u=GMMsource(k).u; % E[x|Wk]
        LSGMMparams(k).v=ni(k,:); % E[y|Wk]
        LSGMMparams(k).E=GMMsource(k).E;% cov(x|Wk)
        LSGMMparams(k).R=diag(lambda(k,:));
    end;
    
    CGMMparams = LSGMMparams;
    
    
    for k=1:pars.m
        CGMMparams(k).v=ni_ls(k,:); % E[y|Wk]
        CGMMparams(k).R=diag(lambda_ls(k,:));
    end;
    
    
    
else
    error('CGMM does not support non-diagonal GMM training');
end